@@ -21,23 +21,67 @@ from fauxfactory.constants import (
 )
 
 
+def codify(data):
+    """
+    Handles unicode compatibility with Python 3
+
+    @type data: str
+    @param data: String to return using unicode.
+
+    @rtype: str
+    @return: String in unicode format.
+    """
+
+    try:
+        result = unicode(data)
+    except NameError:
+        result = data
+
+    return result
+
+
 class FauxFactory(object):
     """
     Generate random data for your tests.
     """
 
+    @classmethod
+    def _is_positive_int(cls, length):
+        """
+        Check that ``length`` argument is a valid integer, greater than zero.
+
+        @type length: int
+        @param length: The desired length of the string
+
+        @rtype: Exception
+        @return: A ``ValueError`` exception
+        """
+
+        # Validate length argument
+        if not isinstance(length, int) or length <= 0:
+            raise ValueError("%s is an invalid \'length\'." % length)
+
     @classmethod
     def generate_string(cls, str_type, length):
         """
         Create of a wide variety of string types, of arbitrary length.
 
         @type str_type: str
-        @param str_type: A valid string type.
+        @param str_type: The desired type of string to be generated.
         @type length: int
         @param length: Length for generated strng.
 
         @rtype: int
         @return: A random string of the type and length specified.
+
+        Valid values for ``str_type`` are as follows:
+
+        * alphanumeric
+        * alpha
+        * latin1
+        * numeric
+        * cjk
+        * utf8
         """
 
         # First lowercase the selected str type
@@ -51,8 +95,10 @@ class FauxFactory(object):
             return cls.generate_latin1(length)
         elif str_type == 'numeric':
             return cls.generate_numeric_string(length)
-        elif str_type == "utf8":
+        elif str_type == "cjk":
             return cls.generate_cjk(length)
+        elif str_type == "utf8":
+            return cls.generate_utf8(length)
         else:
             raise Exception("%s is not a supported string type." % str_type)
 
@@ -69,14 +115,13 @@ class FauxFactory(object):
         """
 
         # Validate length argument
-        if not isinstance(length, int) or length <= 0:
-            raise ValueError("%s is an invalid \'length\'." % length)
+        cls._is_positive_int(length)
 
-        output_string = ''.join(
+        output_string = u''.join(
             random.choice(string.ascii_letters) for i in range(length)
         )
 
-        return unicode(output_string)
+        return codify(output_string)
 
     @classmethod
     def generate_alphanumeric(cls, length=5):
@@ -91,15 +136,14 @@ class FauxFactory(object):
         """
 
         # Validate length argument
-        if not isinstance(length, int) or length <= 0:
-            raise ValueError("%s is an invalid \'length\'." % length)
+        cls._is_positive_int(length)
 
-        output_string = ''.join(
+        output_string = u''.join(
             random.choice(
                 string.ascii_letters + string.digits
             ) for i in range(length))
 
-        return unicode(output_string)
+        return codify(output_string)
 
     @classmethod
     def generate_boolean(cls):
@@ -151,20 +195,17 @@ class FauxFactory(object):
         """
 
         # Validate length argument
-        if not isinstance(length, int) or length <= 0:
-            raise ValueError("%s is an invalid \'length\'." % length)
+        cls._is_positive_int(length)
 
-        cjk_range = []
-        cjk_range = ['4E00', '9FCC']
-        output_array = []
-
-        for i in range(int(cjk_range[0], 16), int(cjk_range[1], 16)):
-            output_array.append(i)
-
-        output_string = ''.join(
-            unichr(random.choice(output_array)) for x in xrange(length))
-
-        return unicode(output_string)
+        # Generate codepoints, then convert the codepoints to a string. The
+        # valid range of CJK codepoints is 0x4E00 - 0x9FCC, inclusive. Python 2
+        # and 3 support the `unichr` and `chr` functions, respectively.
+        codepoints = [random.randint(0x4E00, 0x9FCC) for i in range(length)]
+        try:
+            output = u''.join(unichr(codepoint) for codepoint in codepoints)
+        except NameError:
+            output = u''.join(chr(codepoint) for codepoint in codepoints)
+        return codify(output)
 
     @classmethod
     def generate_date(cls, min_date=None, max_date=None):
@@ -271,9 +312,9 @@ class FauxFactory(object):
         if tlds is None:
             tlds = cls.generate_choice(TLDS)
 
-        email = "%s@%s.%s" % (name, domain, tlds)
+        email = u"%s@%s.%s" % (name, domain, tlds)
 
-        return unicode(email)
+        return codify(email)
 
     @classmethod
     def generate_integer(cls, min_value=None, max_value=None):
@@ -290,8 +331,8 @@ class FauxFactory(object):
         """
 
         # Platform-specific value range for integers
-        _min_value = - sys.maxint - 1
-        _max_value = sys.maxint
+        _min_value = - sys.maxsize - 1
+        _max_value = sys.maxsize
 
         if min_value is None:
             min_value = _min_value
@@ -332,17 +373,14 @@ class FauxFactory(object):
         if not isinstance(words, int) or words < 0:
             raise ValueError(
                 "Cannot generate a string with negative number of words.")
-
-        if not isinstance(paragraphs, int) or paragraphs <= 0:
-            raise ValueError(
-                "Cannot generate a string with negative number of paragraphs.")
+        cls._is_positive_int(paragraphs)
 
         # Original Lorem Ipsum string
         all_words = LOREM_IPSUM_TEXT.split()
         # How many words do we need?
         total_words_needed = words * paragraphs
 
-        quotient = total_words_needed / len(all_words)
+        quotient = int(total_words_needed / len(all_words))
         modulus = total_words_needed % len(all_words)
 
         # Pool of words to use
@@ -370,7 +408,7 @@ class FauxFactory(object):
 
             # Increment positional counter
             start_pos += words
-        return unicode(result.rstrip())
+        return codify(result.rstrip())
 
     @classmethod
     def generate_latin1(cls, length=5):
@@ -386,8 +424,7 @@ class FauxFactory(object):
         """
 
         # Validate length argument
-        if not isinstance(length, int) or length <= 0:
-            raise ValueError("%s is an invalid \'length\'." % length)
+        cls._is_positive_int(length)
 
         range0 = range1 = range2 = []
         range0 = ['00C0', '00D6']
@@ -402,10 +439,14 @@ class FauxFactory(object):
         for i in range(int(range2[0], 16), int(range2[1], 16)):
             output_array.append(i)
 
-        output_string = ''.join(
-            unichr(random.choice(output_array)) for x in xrange(length))
+        try:
+            output_string = u''.join(
+                unichr(random.choice(output_array)) for x in range(length))
+        except NameError:
+            output_string = u''.join(
+                chr(random.choice(output_array)) for x in range(length))
 
-        return unicode(output_string)
+        return codify(output_string)
 
     @classmethod
     def generate_negative_integer(cls):
@@ -436,18 +477,18 @@ class FauxFactory(object):
 
         if ipv6:
             # StackOverflow.com questions: generate-random-ipv6-address
-            ipaddr = ':'.join('{:x}'.format(
+            ipaddr = u':'.join('{:x}'.format(
                 random.randint(0, 2**16 - 1)
             ) for i in range(8))
         else:
             rng = 3 if ip3 else 4
-            ipaddr = ".".join(
+            ipaddr = u".".join(
                 str(random.randrange(0, 255, 1)) for x in range(rng))
 
             if ip3:
-                ipaddr = ipaddr + ".0"
+                ipaddr = ipaddr + u".0"
 
-        return unicode(ipaddr)
+        return codify(ipaddr)
 
     @classmethod
     def generate_mac(cls, delimiter=":"):
@@ -471,7 +512,7 @@ class FauxFactory(object):
             chars[random.randrange(0, len(chars), 1)]+chars[random.randrange(
                 0, len(chars), 1)] for x in range(6))
 
-        return unicode(mac)
+        return codify(mac)
 
     @classmethod
     def generate_numeric_string(cls, length=5):
@@ -486,14 +527,13 @@ class FauxFactory(object):
         """
 
         # Validate length argument
-        if not isinstance(length, int) or length <= 0:
-            raise ValueError("%s is an invalid \'length\'." % length)
+        cls._is_positive_int(length)
 
-        output_string = ''.join(
+        output_string = u''.join(
             random.choice(string.digits) for i in range(length)
         )
 
-        return unicode(output_string)
+        return codify(output_string)
 
     @classmethod
     def generate_positive_integer(cls):
@@ -565,9 +605,46 @@ class FauxFactory(object):
         else:
             tlds = cls.generate_choice(TLDS)
 
-        url = "%s://%s.%s" % (scheme, subdomain, tlds)
+        url = u"%s://%s.%s" % (scheme, subdomain, tlds)
 
-        return unicode(url)
+        return codify(url)
+
+    @classmethod
+    def generate_utf8(cls, length=5):
+        """
+        Returns a random string made up of UTF-8 characters, as per RFC 3629.
+
+        @rtype length: int
+        @param length: Length for random data.
+
+        @rtype: str
+        @return: A random string made up of UTF-8 characters.
+        """
+
+        # Validate length argument
+        cls._is_positive_int(length)
+
+        # Generate codepoints. The valid range of UTF-8 codepoints is
+        # 0x0-0x10FFFF, minus the following: 0xC0-0xC1, 0xF5-0xFF and
+        # 0xD800-0xDFFF. These 2061 invalid codepoints (2 + 11 + 2048) comprise
+        # 0.2% of 0x0-0x10FFFF. Thus, it should be OK to just check for invalid
+        # codepoints and generate new ones if need be.
+        codepoints = []
+        while len(codepoints) < length:
+            codepoint = random.randint(0x0, 0x10FFFF)
+            if (
+                    codepoint not in range(0xC0, 0xC1 + 1)
+                    and codepoint not in range(0xF5, 0xFF + 1)
+                    and codepoint not in range(0xD800, 0xDFFF + 1)):
+                codepoints.append(codepoint)
+
+        # Convert codepoints to characters. Python 2 and 3 support the `unichr`
+        # and `chr` functions, respectively.
+        try:
+            output = u''.join(unichr(codepoint) for codepoint in codepoints)
+        except NameError:
+            output = u''.join(chr(codepoint) for codepoint in codepoints)
+        return codify(output)
 
     @classmethod
     def generate_uuid(cls):
@@ -578,6 +655,6 @@ class FauxFactory(object):
         @return: Returns a string representation for a UUID.
         """
 
-        output_uuid = unicode(uuid.uuid4())
+        output_uuid = codify(str(uuid.uuid4()))
 
         return output_uuid
