@@ -5,8 +5,10 @@ Tests for all string generators
 """
 
 from fauxfactory import FauxFactory
+from sys import version_info
 
 import unittest
+import random
 
 
 class TestStrings(unittest.TestCase):
@@ -40,7 +42,7 @@ class TestStrings(unittest.TestCase):
         @Assert: Alpha string of fixed length is generated
         """
 
-        for length in xrange(2, 12, 2):
+        for length in range(2, 12, 2):
             result = self.factory.generate_alpha(length)
             self.assertEqual(
                 len(result),
@@ -176,7 +178,7 @@ class TestStrings(unittest.TestCase):
         @Assert: Alphanumeric string with fixed length is created
         """
 
-        for length in xrange(2, 12, 2):
+        for length in range(2, 12, 2):
             result = self.factory.generate_alphanumeric(length)
             self.assertEqual(
                 len(result),
@@ -261,7 +263,7 @@ class TestStrings(unittest.TestCase):
         @Assert: CJK string with fixed length is generated
         """
 
-        for length in xrange(2, 12, 2):
+        for length in range(2, 12, 2):
             result = self.factory.generate_cjk(length)
             self.assertEqual(
                 len(result),
@@ -335,10 +337,46 @@ class TestStrings(unittest.TestCase):
         @Assert: CJK string is generated
         """
 
-        result = self.factory.generate_string('utf8', 15)
+        result = self.factory.generate_string('cjk', 15)
         self.assertTrue(
             len(result) > 0, "Empty string was generated")
 
+    def test_generate_utf8_1(self):
+        """
+        @Test: Create a unicode string.
+        @Feature: String Generator
+        @Assert: A unicode string is generated.
+        """
+
+        result = self.factory.generate_string('utf8', 5)
+        if version_info[0] == 2:
+            self.assertTrue(isinstance(result, unicode))
+        else:
+            self.assertTrue(isinstance(result, str))
+
+    def test_generate_utf8_2(self):
+        """
+        @Test: Create a unicode string and specify a length.
+        @Feature: String Generator
+        @Assert: A unicode string of the specified length is generated.
+        """
+
+        length = random.randint(1, 100)
+        self.assertEqual(
+            len(self.factory.generate_string('utf8', length)),
+            length
+        )
+
+    def test_generate_utf8_3(self):
+        """
+        @Test: Create a unicode string and specify an invalid length.
+        @Feature: String Generator
+        @Assert: A ``ValueError`` exception is raised.
+        """
+
+        with self.assertRaises(ValueError):
+            self.factory.generate_string('utf8', 'foo')
+
     def test_generate_latin1_1(self):
         """
         @Test: Create latin1 string of varied length
@@ -357,7 +395,7 @@ class TestStrings(unittest.TestCase):
         @Assert: Latin1 string with fixed length is created
         """
 
-        for length in xrange(2, 12, 2):
+        for length in range(2, 12, 2):
             result = self.factory.generate_latin1(length)
             self.assertEqual(
                 len(result),
@@ -453,7 +491,7 @@ class TestStrings(unittest.TestCase):
         @Assert: Numeric string is created
         """
 
-        for length in xrange(2, 12, 2):
+        for length in range(2, 12, 2):
             result = self.factory.generate_numeric_string(length)
             self.assertEqual(
                 len(result),
